use serenity;

use std::fs;
use std::sync::Arc;

use super::super::prelude::*;

pub struct Remove {
    media: Arc<MediaData>,
}

impl Remove {
    pub fn new(media_data: Arc<MediaData>) -> Remove {
        Remove { media: media_data }
    }
}

impl serenity::framework::standard::Command for Remove {
    #[allow(unreachable_code, unused_mut)]
    fn execute(
        &self,
        mut _ctx: &mut serenity::client::Context,
        msg: &serenity::model::channel::Message,
        mut _args: serenity::framework::standard::Args,
    ) -> ::std::result::Result<(), serenity::framework::standard::CommandError> {
        let song_name = self.media.name_mut();
        let name: String = song_name.borrow().clone();

        if !name.is_empty() {
            match fs::remove_file(&name) {
                Ok(_) => print_error!(
                    msg.channel_id
                        .say(format!("Remove current song ({}) from local disk", name))
                ),
                Err(err) => print_error!(msg.channel_id.say(format!(
                    "Error removing file ({}): {:?}",
                    name,
                    err.kind()
                ))),
            };
        }

        Ok(())
    }
}
